<?php
declare(strict_types=1);

namespace CakeProducts\Model\Table;

use Cake\Datasource\EntityInterface;
use Cake\Datasource\ResultSetInterface;
use Cake\ORM\Association\BelongsTo;
use Cake\ORM\Behavior\TimestampBehavior;
use Cake\ORM\Query\SelectQuery;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use CakeProducts\Model\Entity\ExternalProductCatalog;
use Closure;
use Psr\SimpleCache\CacheInterface;

/**
 * ExternalProductCatalogs Model
 *
 * @property ProductCatalogsTable&BelongsTo $ProductCatalogs
 *
 * @method ExternalProductCatalog newEmptyEntity()
 * @method ExternalProductCatalog newEntity(array $data, array $options = [])
 * @method array<ExternalProductCatalog> newEntities(array $data, array $options = [])
 * @method ExternalProductCatalog get(mixed $primaryKey, array|string $finder = 'all', CacheInterface|string|null $cache = null, Closure|string|null $cacheKey = null, mixed ...$args)
 * @method ExternalProductCatalog findOrCreate($search, ?callable $callback = null, array $options = [])
 * @method ExternalProductCatalog patchEntity(EntityInterface $entity, array $data, array $options = [])
 * @method array<ExternalProductCatalog> patchEntities(iterable $entities, array $data, array $options = [])
 * @method ExternalProductCatalog|false save(EntityInterface $entity, array $options = [])
 * @method ExternalProductCatalog saveOrFail(EntityInterface $entity, array $options = [])
 * @method iterable<ExternalProductCatalog>|ResultSetInterface<ExternalProductCatalog>|false saveMany(iterable $entities, array $options = [])
 * @method iterable<ExternalProductCatalog>|ResultSetInterface<ExternalProductCatalog> saveManyOrFail(iterable $entities, array $options = [])
 * @method iterable<ExternalProductCatalog>|ResultSetInterface<ExternalProductCatalog>|false deleteMany(iterable $entities, array $options = [])
 * @method iterable<ExternalProductCatalog>|ResultSetInterface<ExternalProductCatalog> deleteManyOrFail(iterable $entities, array $options = [])
 *
 * @mixin TimestampBehavior
 */
class ExternalProductCatalogsTable extends Table
{
    /**
     * Initialize method
     *
     * @param array<string, mixed> $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void
    {
        parent::initialize($config);

        $this->setTable('external_product_catalogs');
        $this->setDisplayField('base_url');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsToMany('ProductCatalogs', [
            'through' => 'CakeProducts.ExternalProductCatalogsProductCatalogs',
            'className' => 'CakeProducts.ProductCatalogs',
        ]);
    }

    /**
     * Default validation rules.
     *
     * @param Validator $validator Validator instance.
     * @return Validator
     */
    public function validationDefault(Validator $validator): Validator
    {
        $validator
            ->uuid('product_catalog_id')
            ->notEmptyString('product_catalog_id');

        $validator
            ->scalar('base_url')
            ->maxLength('base_url', 255)
            ->requirePresence('base_url', 'create')
            ->notEmptyString('base_url');
//            ->url('base_url');

        $validator
            ->scalar('api_url')
            ->maxLength('api_url', 255)
            ->requirePresence('api_url', 'create')
            ->notEmptyString('api_url');
//            ->url('api_url');

        $validator
            ->dateTime('deleted')
            ->allowEmptyDateTime('deleted');

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param RulesChecker $rules The rules object to be modified.
     * @return RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker
    {
        $rules->add($rules->existsIn(['product_catalog_id'], 'ProductCatalogs'), ['errorField' => 'product_catalog_id']);

        return $rules;
    }
}
